
# q-expansion of newform 1274.2.f.r, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1274
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[885, [0, -1]], [197, [1, 0]]]
aps_data = [[0, 1], [1, -1], [0, -3], [0, 0], [-6, 6], [-1, 0], [-3, 3], [0, 2], [0, 0], [6, 0], [-4, 4], [0, 7], [0, 0], [-1, 0], [0, 3], [0, 0], [-6, 6], [0, 8], [-14, 14], [-3, 0], [2, -2], [0, -8], [-12, 0], [0, -6], [10, 0], [-12, 12], [0, -4], [0, -12], [7, -7], [-6, 0], [20, 0], [0, -21], [0, 0], [13, 0], [0, 6], [-17, 17], [14, -14], [0, 16], [0, 0], [0, 0], [-3, 3], [-20, 0], [0, 18], [4, -4], [3, 0], [2, -2], [-13, 0], [19, 0], [0, 0], [0, -13], [0, 27], [15, 0], [-10, 10], [-24, 0], [0, 9], [12, -12], [24, -24], [0, 11], [28, -28], [-6, 0], [-4, 4], [-21, 0], [-2, 0], [-30, 30], [0, -1], [0, 6], [0, -8], [23, 0], [-3, 3], [19, 0], [24, -24], [0, 0], [26, -26], [0, 4], [20, 0], [0, 21], [6, -6], [0, -34], [0, -36], [32, -32], [-9, 0], [17, 0], [33, -33], [25, 0], [0, 26], [0, -21], [6, 0], [0, 10], [-9, 0], [-40, 0], [0, 36], [-21, 21], [16, -16], [-9, 0], [0, 40], [30, 0], [0, -18], [-9, 9], [0, 20], [0, -11], [17, 0], [-3, 3], [39, -39], [0, -15], [-5, 5], [38, -38], [-24, 0], [0, 18], [-6, 6], [19, 0], [0, 14], [-38, 38], [-24, 0], [-28, 28], [29, 0], [18, -18], [-14, 0], [-6, 6], [0, 0], [-36, 0], [-22, 22], [-19, 0], [0, 48], [-24, 24], [0, 8], [0, 0], [0, -26], [0, 6], [10, 0], [0, 23], [-20, 20], [-9, 0], [0, 40], [-16, 0], [0, -6], [-32, 0], [-39, 39], [-40, 40], [42, 0], [33, -33], [-20, 0], [0, 3], [-14, 14], [18, 0], [38, -38], [0, 0], [37, 0], [-42, 42], [0, -4], [0, 45], [0, 13], [-21, 0], [29, 0], [0, 0], [37, -37], [30, 0], [0, 16], [0, 36], [34, 0], [-21, 21], [0, -6], [15, 0], [-31, 0], [0, -3], [54, -54], [39, -39], [-2, 2], [-46, 46], [38, 0], [0, 6], [-30, 30], [-23, 0], [0, 24], [-8, 8], [14, -14], [33, 0], [44, 0], [54, -54], [-20, 0], [0, 8], [0, -8], [-12, 0], [-22, 0], [0, -18], [36, -36], [54, -54], [-38, 38], [-10, 10], [0, 4], [24, -24], [0, 29], [36, 0], [0, -65], [0, -30], [3, -3], [-54, 54], [40, -40], [0, -44], [-48, 0], [0, -54], [-21, 21], [-8, 0], [0, -13], [17, -17], [24, 0], [-6, 6], [0, -7], [0, -12], [42, 0], [-31, 31], [0, -38], [-6, 0], [-64, 0], [0, -18], [57, -57], [0, -28], [4, -4], [-27, 27], [0, -12], [-21, 0], [0, 4], [-41, 0], [0, -60], [0, 7], [-9, 0], [-46, 0], [0, 15], [12, -12], [0, 62], [0, -30], [58, -58], [-52, 52], [32, 0], [-51, 51], [-53, 0], [-33, 33], [0, -22], [0, 33], [48, 0], [-30, 0], [66, -66], [0, -25], [32, -32], [0, 61], [42, 0], [0, 12], [43, 0], [-36, 36], [22, -22], [66, 0], [41, 0], [0, -6], [3, -3], [1, 0], [48, -48], [0, 30], [16, -16], [-46, 46], [36, 0], [0, -34], [-44, 44], [-21, 0], [-49, 49], [-2, 0], [-24, 24], [0, 35], [-39, 0], [24, 0], [-4, 0], [-12, 12], [0, 68], [-2, 2], [26, -26], [0, -32], [-14, 0], [0, -16], [0, 6], [-62, 62], [0, -20], [0, -63], [36, -36], [67, -67], [42, 0], [-26, 0], [0, 32], [0, -18], [-65, 65], [-57, 0], [71, -71], [-42, 0], [12, -12], [15, -15], [0, 0], [42, 0], [26, 0], [60, -60], [0, -46], [12, 0], [0, -15], [82, 0], [0, 23], [0, -9], [-38, 38]]
